import josx.platform.rcx.*;

public class Circuit implements SerialListener {
	private int greenLight, tolerance = 10;
	private byte continueMsg = (byte) 101, barrelOnGreenMsg = (byte) 212;
	private boolean onGreenOrOnBlack, drive, alive, direction;
	public boolean continueOK, barrelOnGreenOK;
	public int turnTime, greenMaxCount, fastDriveTime = 60;
	
	public int rotationTime;
	public final int ROTATION_POWER = 7;
	public final int DRIVE_POWER = 3;
	public final int DRIVE_HIGH_POWER = 7;
	

	public void PRGM() {
		try {
			Button.PRGM.waitForPressAndRelease();
		} catch (Exception e) {
		}
		
	}
	
	public void initRotation() {
		int time = getTime();
		rotateUntilOnLine();		
		turnTime = getTime() - time + 100;
		motorStop();
		try {
			Button.VIEW.waitForPressAndRelease();
		} catch (Exception e) {
		}
	}
	
	public void initFastDriveTime() {
		LCD.showNumber(fastDriveTime);
		
		while (!Button.PRGM.isPressed()) {
			if (Button.VIEW.isPressed()) {
				fastDriveTime += 10;
				if (fastDriveTime == 210) fastDriveTime = 10;
				LCD.showNumber(fastDriveTime);
				sleep(200);
			}
		}
		while (Button.PRGM.isPressed());
	}

	public void driveFastOnLine() {
		int time = getTime() + fastDriveTime * 1000;
		while (getTime() < time)
			driveOnLine(true);
		
		// Beep-Ton, um Uebergang auf den
		// anderen Zustand zu signalisieren
		Sound.beep();
	}
	
	public Circuit() {
		Serial.setRangeLong();

		// Lichtsensor initialisieren
		Sensor.S2.setTypeAndMode(
				SensorConstants.SENSOR_TYPE_LIGHT,
				SensorConstants.SENSOR_MODE_RAW);
		Sensor.S2.setPreviousValue(0);
		Sensor.S2.activate();
		
		readGreenLight();
		PRGM();

		// Button.PRGM.addButtonListener(this);
		Serial.addSerialListener(this);
		
		alive = true;
		direction = true;
		continueOK = false;
		barrelOnGreenOK = false;
		
		
	}
	
	// Roboter stirbt, wenn Button gedrueckt wird (ButtonListener-Methode)
	public void buttonPressed(Button b) {
		alive = false;
	}

	// wird nicht verwendet (ButtonListener-Methode)
	public void buttonReleased(Button b) {
	}

	// Linie entlang fahren bis zu einer grossen gruenen
	//  Flaeche (kleine gruene Flaechen werden ignoriert)
	public void driveOnLineUntilHasBarrel() {
		int greenCounter = 0;
		boolean stop = false;
		while (!stop && alive) {
			driveOnLine(false);
			greenCounter = isOnGreen() ? greenCounter + 1 : 0;
			stop = greenCounter >= 3;
			
			if (stop) {
				driveBack(50);
				setMotorDirection();
				int t = getTime() + 200;
				motorsPower(ROTATION_POWER);
				while (getTime() < t && alive);
				stop = isOnGreen();
				if (stop) {
					reverseMotor();
					t = getTime() + 200;
					motorsPower(ROTATION_POWER);
					while (getTime() < t && alive);
					stop = isOnGreen();
				}
			}
		}
	}
	
	public void driveOnLineUntilOnGreen() {
		int greenCounter = 0;
		while (greenCounter < greenMaxCount) {
			driveOnLine(true);
			greenCounter = isOnGreen() ? greenCounter + 1 : 0;
		}
	}

	// Methode zum Folgen der Linie
	private void driveOnLine(boolean back) {
		onGreenOrOnBlack = !isOnWhite();
		// geradeaus fahren, wenn auf der Linie
		if (onGreenOrOnBlack && !drive) {
			drive = true;
			motorsPower(back ? DRIVE_HIGH_POWER : DRIVE_POWER);
			Motor.A.forward();
			Motor.C.forward();
		}

		// Linie wurde verlassen
		else if (!onGreenOrOnBlack && drive) {
			drive = false;
			setMotorDirection();
			for (int i = 1; !onGreenOrOnBlack; i += 2) {
				rotateTime(i * rotationTime);
				reverseMotor();
			}
			reverseMotor();
		}
	}
	
	// zurueckfahren
	public void driveBack(int time) {
		motorsPower(DRIVE_POWER);
		Motor.A.backward();
		Motor.C.backward();
		sleep(time);
	}
	
	// zurueckfahren
	public void driveForward(int time) {
		motorsPower(DRIVE_POWER);
		Motor.A.forward();
		Motor.C.forward();
		sleep(time);
	}
	
	
	
	// abschliessende Befehle zum Abschalten des Roboters
	public void passivate() {
		Sensor.S2.passivate();
		Motor.A.flt();
		Motor.C.flt();
		try {
			Button.RUN.waitForPressAndRelease();
		} catch (Exception e) {
		}
	}
	
	// Roboter dreht sich, bis er wieder auf der Linie ist
	public void rotateUntilOnLine() {
		setMotorDirection();
		motorsPower(ROTATION_POWER);
		sleep(1000);
		while (!isOnLine() && alive);
	}
	
	public void rotateUntilOnLineAndDrive() {
		setMotorDirection();
		int time = getTime() + turnTime;
		while (getTime() <= time && alive);
		driveForward(2000);
	}

	// Roboter dreht sich fuer eine bestimmte Zeit
	private void rotateTime(int time) {
		int t = getTime() + time;
		motorsPower(ROTATION_POWER);
		while (getTime() < t && isOnWhite() && alive);

		onGreenOrOnBlack = !isOnWhite() || Button.RUN.isPressed();
	}
	
	// Zeit als Int-Wert zurueckgeben
	private int getTime() {
		return (int) System.currentTimeMillis();
	}

	// Lenkrichtung aendern (Richtung beider Motoren umdrehen)
	public void reverseMotor() {
		direction = !direction;
		if (direction) {
			Motor.A.forward();
			Motor.C.backward();

		} else {
			Motor.C.forward();
			Motor.A.backward();
		}
	}

	// die Richtung der Motoren fuer die Drehung setzen
	private void setMotorDirection() {
		if (direction) {
			Motor.A.forward();
			Motor.C.flt();
			sleep(100);
			Motor.C.backward();

		} else {
			Motor.C.forward();
			Motor.A.flt();
			sleep(100);
			Motor.A.backward();
		}
	}

	// pruefen, ob Roboter auf der Linie
	public boolean isOnLine() {
		return  Sensor.S2.readRawValue() >= greenLight + tolerance;
	}
	
	public boolean isOnGreen() {
		int val = Sensor.S2.readRawValue();
		return val >= greenLight - tolerance && val <= greenLight + tolerance;
	}
	
	public boolean isOnWhite() {
		return Sensor.S2.readRawValue() <= greenLight - tolerance;
	}
	
	// Gruen-Wert ermitteln
	public void readGreenLight() {
		motorsPower(DRIVE_POWER);
		Motor.A.forward();
		Motor.C.forward();
		
		int sum = 0, darklineDatacount = 500;
		for (int i = 0; i <= darklineDatacount; i++) {
			sum += Sensor.S2.readValue();
		}
		greenLight = sum / darklineDatacount;
		LCD.showNumber(greenLight);
	
		motorStop();
	}
	
	private void motorStop() {
		Motor.A.stop();
		Motor.C.stop();
	}

	// Geschwindigkeit der beider Motoren setzen
	public void motorsPower(int p) {
		Motor.A.setPower(p);
		Motor.C.setPower(p);
	}

	public void sleep(int aMilliseconds) {
		try {
			Thread.sleep(aMilliseconds);
		} catch (Exception e) {
		}
	}
	
	public void sendContinue() {
		sendByte(continueMsg);
	}

	public void sendBarrelOnGreen() {
		sendByte(barrelOnGreenMsg);
	}
	
	public void waitForContinue() {
		// Motoren anhalten und solange warten bis die
		// Tonne auf dem gruenem Bereich eingesammelt wurde
		motorStop();
		while (!continueOK);
		continueOK = false;
	}
	
	public void waitForBarrelOnGreen() {
		// Motoren anhalten und solange warten bis
		// eine Tonne zum gruenem Bereich gebracht wurde
		motorStop();
		while (!barrelOnGreenOK);
		barrelOnGreenOK = false;
	}
	
	// Byte senden
	private void sendByte(byte b) {
		byte[] packet = { (byte) 0xf7, (byte) 0x00 };
		packet[1] = b;
		Serial.sendPacket(packet, 0, 2);
		Sound.beepSequence();
	}

	// Byte empfangen (SerialListener-Methode)
	public void packetAvailable(byte[] packet, int length) {
		byte answer = packet[1];
		if (answer == continueMsg) continueOK = true;
		else if (answer == barrelOnGreenMsg) barrelOnGreenOK = true;
		Sound.twoBeeps();
	}
}
